package org.example.controller;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.servlet.annotation.WebServlet;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.example.model.InvalidNewsException;
import org.example.model.NewsAgency;
import org.example.model.NewsItem;
import org.example.model.NewsNotFoundException;

import java.io.IOException;
import java.util.List;

@WebServlet("/api/news/*")
public class NewsServlet extends HttpServlet {

	private static final String JSON_MEDIA_TYPE = "application/json";

	private final NewsAgency newsAgency = NewsAgency.getInstance();
	private final ObjectMapper objectMapper = ObjectMapperFactory.createObjectMapper();

	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
		List<NewsItem> news = newsAgency.getNews();
		String acceptHeader = request.getHeader("Accept");
		if (acceptHeader != null && !acceptHeader.contains(JSON_MEDIA_TYPE)) {
			response.setStatus(HttpServletResponse.SC_NOT_ACCEPTABLE);
		} else {
			response.setStatus(HttpServletResponse.SC_OK);
			response.setContentType(JSON_MEDIA_TYPE);
			objectMapper.writeValue(response.getWriter(), news);
		}
	}

	@Override
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
		try {
			NewsItem newsItem = objectMapper.readValue(request.getReader(), NewsItem.class);
			newsAgency.postNews(newsItem);
			response.setStatus(HttpServletResponse.SC_CREATED);
			response.setContentType(JSON_MEDIA_TYPE);
			objectMapper.writeValue(response.getWriter(), newsItem);
		} catch (JsonParseException | InvalidNewsException ex) {
			response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
		}
	}

	@Override
	public void doDelete(HttpServletRequest request, HttpServletResponse response) {
		String path = request.getPathInfo();
		if (path == null || path.equals("/")) {
			response.setStatus(HttpServletResponse.SC_METHOD_NOT_ALLOWED);
			return;
		}
		try {
			int id = Integer.parseInt(path.substring(1));
			newsAgency.deleteNews(id);
			response.setStatus(HttpServletResponse.SC_NO_CONTENT);
		} catch (NumberFormatException | NewsNotFoundException ex) {
			response.setStatus(HttpServletResponse.SC_NOT_FOUND);
		}
	}
}
